/* parse_igmp.c
 * Stuff for dealing with igmp packets.
 */

#include "weedlog.h"

struct igmppacket {
  struct wl_iphdr {
#if __BYTE_ORDER == __LITTLE_ENDIAN
    u_char	ihl:4,
		version:4;
#elif __BYTE_ORDER == __BIG_ENDIAN
    u_char	version:4,
		ihl:4;
#else
#error	"Please fix <asm/byteorder.h>"
#endif
    u_char	tos;
    u_short	tot_len;
    u_short	id;
    u_short	frag_off;
    u_char	ttl;
    u_char	protocol;
    u_short	check;
    u_long	saddr;
    u_long	daddr;
  }ip;
  struct wl_igmphdr {
    u_char	type;
    u_char	code;
    u_short	csum;
    u_long	group;
  }igmp;
};

int parse_igmp(const u_char *packbuff, int verbose) {
  int pktsize=0;
  char newbuf[512], suspip[24], psizebuf[24], tempflags[128], ipflags[128],
      saddress[82], daddress[88], pkttype[64], csums[64], miscstuff[200];
  struct igmppacket *pkt = (struct igmppacket*)packbuff;

  if(pkt->ip.ihl != 5)
    snprintf(suspip, sizeof(suspip), " Suspicious IP Options: ");
  else
    snprintf(suspip, sizeof(suspip), " ");

  if((pktsize = ntohs(pkt->ip.tot_len)) > 1024)
    snprintf(psizebuf, sizeof(psizebuf), "%4.02f KBytes", (float)pktsize / 1024.0);
  else
    snprintf(psizebuf, sizeof(psizebuf), "%4.02f Bytes", (float)pktsize);

  if(pkt->ip.saddr) {
    struct in_addr s;
    s.s_addr = pkt->ip.saddr;
    snprintf(saddress, sizeof(saddress), "%s[%s]",
	hostlookup(pkt->ip.saddr), inet_ntoa(s));
  } else {
    snprintf(saddress, sizeof(saddress), "nowhere it would seem");
  }
  if(pkt->ip.daddr) {
    struct in_addr d;
    d.s_addr = pkt->ip.daddr;
    snprintf(daddress, sizeof(daddress), " to %s[%s],",
	hostlookup(pkt->ip.daddr), inet_ntoa(d));
  } else {
    snprintf(daddress, sizeof(daddress), ",");
  }

  if(verbose >= 3) {
    snprintf(ipflags, sizeof(ipflags), "  IP_FLAGS:");
    if(pkt->ip.version > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s VER=%i", ipflags,
	  pkt->ip.version);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
    if(pkt->ip.protocol > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s PROTO=%i", ipflags,
	  pkt->ip.protocol);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
    if(pkt->ip.frag_off > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s FRAG_OFF=%i", ipflags,
	  pkt->ip.frag_off);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
    if(pkt->ip.tos > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s TOS=%i", ipflags,
	  pkt->ip.tos);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
    if(pkt->ip.id > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s ID=%i", ipflags,
	  pkt->ip.id);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
    if(pkt->ip.ttl > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s TTL=%i", ipflags,
	  pkt->ip.ttl);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
    if(pkt->ip.check > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s CHECK=%i", ipflags,
	  pkt->ip.check);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
  } else snprintf(ipflags, sizeof(ipflags), " ");

  snprintf(pkttype, sizeof(pkttype), "Type %u (code %u)", pkt->igmp.type,
    pkt->igmp.code);

  if(pkt->igmp.group) {
    struct in_addr g;
    g.s_addr = pkt->igmp.group;
    snprintf(miscstuff, sizeof(miscstuff), "  Group: %s[%s] ",
	hostlookup(pkt->igmp.group), inet_ntoa(g));
/*    snprintf(miscstuff, sizeof(miscstuff), "  Group: %s ",
	inet_ntoa(g));*/
  } else snprintf(miscstuff, sizeof(miscstuff), " ");

  if(verbose >= 2)
    snprintf(csums, sizeof(csums), " Checksum: %u", pkt->igmp.csum);
  else snprintf(csums, sizeof(csums), " ");

  snprintf(newbuf, sizeof(newbuf), "IGMP:%s%s packet from %s%s Size: %s"
      "%s%s%s", suspip, pkttype, saddress, daddress, psizebuf, miscstuff, csums,
      ipflags);
  print_log("%s\n", newbuf);

  return 0;
}
